import os
import boto3

from circles_local_aws_s3_storage_python.StorageInterface import StorageInterface
from circles_local_aws_s3_storage_python.StorageDB import StorageDB
from circles_local_aws_s3_storage_python import constants

class AwsS3Storage(StorageInterface):

    def __init__(self, bucket_name, region):
        self.region = region
        self.bucket_name = bucket_name
        self.database = StorageDB()
        aws_access_key_id=os.getenv("AWS_ACCESS_KEY_ID")
        aws_secret_access_key=os.getenv("AWS_SECRET_ACCESS_KEY")
        self.client = boto3.client('s3',
                                   aws_access_key_id=aws_access_key_id,
                                   aws_secret_access_key=aws_secret_access_key)

    # uploads file to S3

    def upload_file(self, local_path, filename, remote_path, created_user_id, url = None):
        read_binary = 'rb'
        with open(local_path, read_binary) as file_obj:
            file_contents = file_obj.read()

        # Upload the file to S3 with the CRC32 checksum
        response = self.client.put_object(
            Bucket=self.bucket_name,
            Key=remote_path+filename,
            Body=file_contents,
            ChecksumAlgorithm='crc32'
        )
        if 'ETag' in response:
            id = self.database.uploadToDatabase(
                remote_path, filename, self.region, created_user_id, constants.STORAGE_TYPE_ID, constants.FILE_TYPE_ID, constants.EXTENSION_ID, url)  # Constants needs to be replaced by parameter
            return id
        return None

    # download a file from s3 to local_path
    def download_file(self, remote_path, local_path):
        print(self.bucket_name,remote_path,local_path)
        self.client.download_file(self.bucket_name, remote_path, local_path)

    # logical delete
    def delete_file(self, remote_path, filename, updated_user_id):
        self.database.logicalDelete(
            remote_path, filename, self.region, updated_user_id)




import sys
import boto3
import os
import unittest
from dotenv.main import load_dotenv
sys.path.append(os.getcwd())
from circles_local_aws_s3_storage_python.AWSStorage import AwsS3Storage # noqa: E402
from circles_local_aws_s3_storage_python.StorageDB import StorageDB # noqa: E402
load_dotenv()



PROFILE_ID = 1


class s3_test(unittest.TestCase):

    def setUp(self) -> None:
        self.s3_resource = boto3.resource('s3')
        self.awsS3 = AwsS3Storage(
            os.getenv("BUCKET_NAME"), os.getenv("REGION"))
        self.test_file_contents = b'this it a file test!'
        self.database = StorageDB()

    def test_upload(self):
        cwd = os.getcwd()
        filepath = os.path.join(cwd, 'tests/test.txt')
        functionId = self.awsS3.upload_file(
            filepath, 'test.txt', 'python/', PROFILE_ID)
        s3_object = self.s3_resource.Object(
            os.getenv("BUCKET_NAME"), 'python/test.txt')
        s3_file_contents = s3_object.get()['Body'].read()
        self.assertEqual(s3_file_contents, self.test_file_contents)
        actualId = self.database.getLastId()
        self.assertEqual(functionId, actualId)

    def test_download(self):
        cwd = os.getcwd()
        self.awsS3.download_file(
            'python/test.txt', cwd+'/test.txt')
        assert os.path.isfile(
            cwd+'/test.txt')

    def test_logical_delete(self):
        self.awsS3.delete_file('python/', 'test.txt', PROFILE_ID)
        result = self.database.getEndTimeStampFromDB(
            'python/', 'test.txt', os.getenv("REGION"))
        print(result)
        self.assertIsNotNone(result[0])


if __name__ == '__main__':
    unittest.main()



import os
import mysql.connector
from dotenv.main import load_dotenv
load_dotenv()


class file_type_db:
    def __init__(self) -> None:
        self.mydb = mysql.connector.connect(
            host=os.getenv("RDS_HOSTNAME"),
            user=os.getenv("RDS_USERNAME"),
            password=os.getenv("RDS_PASSWORD"),
            database="storage"
        )
        self.cursor = self.mydb.cursor()

    def select_from_DB(self, select_stmt, select_data):
        self.cursor.execute(select_stmt, [select_data])
        folder = self.cursor.fetchone()[0]
        return folder



import os
import mysql.connector
from dotenv.main import load_dotenv
load_dotenv()


class StorageDB:
    def __init__(self) -> None:
        self.mydb = mysql.connector.connect(
            host=os.getenv("RDS_HOSTNAME"),
            user=os.getenv("RDS_USERNAME"),
            password=os.getenv("RDS_PASSWORD"),
            database="storage"
        )
        self.cursor = self.mydb.cursor(buffered=True)

    def uploadToDatabase(self, file_path, filename,  region, created_user_id, storage_type_id, file_type_id, extension_id, url = None) -> int:
        add_storage = ("INSERT INTO storage.storage_table "
                       "(path, filename, region, url, created_user_id, updated_user_id, storage_type_id, file_type_id, file_extension_id) "
                       "VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s)")
        storage_data = (file_path, filename, region, url, created_user_id,
                        created_user_id, storage_type_id, file_type_id, extension_id)
        self.cursor.execute(add_storage, storage_data)
        self.mydb.commit()
        select_stmt = "SELECT LAST_INSERT_ID()"
        self.cursor.execute(select_stmt)
        last_insert_id = self.cursor.fetchone()[0]
        return int(last_insert_id)

    def closeConnection(self):
        if (self.mydb != None):
            self.mydb.close

    def logicalDelete(self, remote_path, filename, region, updated_user_id) -> int:
        select_stmt = "SELECT storage_id FROM storage.storage_view WHERE path = %s AND filename = %s AND region = %s"
        select_data = (remote_path, filename, region)
        self.cursor.execute(select_stmt, select_data)
        id = self.cursor.fetchone()[0]
        self.cursor.nextset()
        update_stmt = "UPDATE storage.storage_table SET end_timestamp = NOW(), updated_timestamp = NOW(), updated_user_id = %s WHERE path = %s AND filename = %s AND region = %s"
        update_data = (updated_user_id, remote_path, filename, region)
        self.cursor.execute(update_stmt, update_data)
        self.mydb.commit()
        return int(id)

    def getEndTimeStampFromDB(self, remote_path, filename, region):
        update_stmt = "SELECT end_timestamp FROM storage.storage_view WHERE path = %s AND filename = %s AND region = %s"
        update_data = (remote_path, filename, region)
        self.cursor.execute(update_stmt, update_data)
        return self.cursor.fetchone()

    def getLastId(self) -> int:
        select_stmt = "SELECT storage_id FROM storage.storage_view ORDER BY storage_id DESC LIMIT 1"
        self.cursor.execute(select_stmt)
        last_insert_id = self.cursor.fetchone()[0]
        return int(last_insert_id)
